const { match } = require('assert');
const { publicDecrypt } = require('crypto');
const CNF_FS = require('fs');

/**
 * // CNF = CoreNodeFoncts
 * const CNF_FS = require('fs');
 * 
 * @param {CNF_HTTP.ServerResponse} response - The response entered in this parameter must come from your server.
 * @param {String} filePath 
 * For security reasons you must create a sub-folder to put your files. 
 * The links in your server must be absolute to work with this function. 
 * The file path must be existing in your child architecture of the server 
 * and must include the name of the folder containing the architecture 
 * intended for website users.
 * @param {String} contentType - The parameter "contentType" must be contain only a MIME type.
 * @param {Number} status - The "status" parameter must contain only a number. You should only use HTTP status codes here.
 * @returns 
 */
 function sendFile(response, filePath, contentType, status = 200) {
    try {
        file = CNF_FS.readFileSync("." + filePath);
        response.setHeader("Content-Type", contentType);
        response.writeHead(status);
        return response.end(file);
    } catch (error) {
        if (CNF_FS.existsSync('./PUBLIC/ERREUR/404.html')) {
            return response.end(CNF_FS.readFileSync('./PUBLIC/ERREUR/404.html'));
        } else {
            response.setHeader("Content-Type", "text/html");
            response.writeHead(404);
            response.end("Erreur 404");
        }
    }
}

/**
 *  
 * This function requires the presence of the function loadPicturesFromFile to load 
 * dinamically load the links for the pictures.
 * 
 * @param {CNF_HTTP.ServerResponse} response 
 * @param {String} filePath 
 * For security reasons you must create a sub-folder to put your files. 
 * The links in your server must be absolute to work with this function. 
 * The file path must be existing in your child architecture of the server 
 * and must include the name of the folder containing the architecture 
 * intended for website users.
 * @returns 
 */
function sendHTMLFile(response, filePath){
    let file;
    let sharedFiles = {};
    sharedFiles["head"] = CNF_FS.readFileSync('./PUBLIC/HTML5/Shared/Head.html', {encoding: "UTF-8"}).toString();
    sharedFiles["header"] = CNF_FS.readFileSync('./PUBLIC/HTML5/Shared/Header.html', {encoding: "UTF-8"}).toString();
    sharedFiles["footer"] = CNF_FS.readFileSync('./PUBLIC/HTML5/Shared/Footer.html', {encoding: "UTF-8"}).toString();
    sharedFiles["contact"] = CNF_FS.readFileSync('./PUBLIC/HTML5/Shared/Contact_Button.html', {encoding: "UTF-8"}).toString();

    try {
        let arrayPages = [
            ["Accueil.html","Accueil"],
            ["About_Us.html","A_Propos"],
            ["Gallery.html","Galerie"]
        ]

        let arrayLink = [
            ["/IMG/Accueil/Slider/Active",  "D", "photo"],
            ["/TXT/ContenuAccueil.txt", "F", "contenuAccueil"],
            ["/IMG/A_Propos/Haut",          "D", "photoH"],
            ["/IMG/A_Propos/Trombinoscope", "D",  "photoB"],
            ["/TXT/ContenuA_Propos.txt", "F", "contenuAPropos"],
            ["/IMG/Galerie/Active",         "D", "photo"]
        ];

        let regexTXT = /^(\/TXT)+((\/)+([a-zA-z]{1,}){1,})+(\.)+(txt)$/;
        let regexIMG = /^(\/IMG)+((\/)+([a-zA-z]{1,}){1,})+(\w){1,}$/;
        let resultatTXT = [];
        let resultatIMG = [];
        let linkForPage = [];
         
        // Vérification de la bonne écriture des liens provenant du dossier IMG et TXT depuis l'arborescence
        // Construction d'un tableau de lien séparant les IMG et TXT
        arrayLink.forEach(link => {
            if (link[0].match(regexIMG)){
                resultatIMG.push(link[0]);
            }else if (link[0].match(regexTXT)){
                resultatTXT.push(link[0]);
            }else{
                console.log("Erreur Folder");
            }
        });
        
        function buildArrayLinksForAskedPage (resultatTXT, resultatIMG, value){
            for (let j=0;j<resultatTXT.length;j++){
                // Vérification que le lien contenu dans resultatTXT contient un lien destiné à la page demandé
                // Si c'est le cas, le lien vers une ressource TXT est ajouté aux tableau regroupant tous les liens pour une page.
                if(resultatTXT[j].includes(value)){
                    linkForPage.push(resultatTXT[j]);
                }
            }
            for (let j=0;j<resultatIMG.length;j++){
                // Vérification que le lien contenu dans resultatTXT contient un lien destiné à la page demandé
                // Si c'est le cas, le lien vers une ressource TXT est ajouté aux tableau regroupant tous les liens pour une page.
                if(resultatIMG[j].includes(value)){
                    linkForPage.push(resultatIMG[j] );
                }
            }
            return linkForPage
        }

        // Chargement des parties communes des pages Web
        file = CNF_FS.readFileSync("." + filePath, {encoding: "UTF-8"});
        file = file.toString().supplant(sharedFiles);

        if (filePath.includes(arrayPages[0][0])){
            // Page d'accueil
            // console.log(arrayPages[0][1] + " est demandée");
            // Récupération lien depuis liste des contenus dynamiques affich" dans le tableau arrayLink
            buildArrayLinksForAskedPage (resultatTXT, resultatIMG, arrayPages[0][1]);
            // Chargement du contenu dynamique selon la liste de lien contruite précédemment

            linkForPage.forEach(link => { 
                if (link.match(regexTXT)) {
                    console.log(link);
                    let text = {};
                    text[arrayLink[1][2]] = CNF_FS.readFileSync(`./PUBLIC/${link}`, {encoding: "UTF-8"}).toString();
                    file = file.toString().supplant(text);
                } else if (link.match(regexIMG)) {
                    let ListFileInStorageFolder = CNF_FS.readdirSync(`./PUBLIC${link}`, {encoding: "UTF-8"});
                    if (ListFileInStorageFolder != ""){
                        let ArrayLinksFileIMG = [];
                        ListFileInStorageFolder.forEach(fileName => {
                            ListFileInStorageFolder.fileName = `${link}/${fileName}`;
                            ArrayLinksFileIMG.push(ListFileInStorageFolder.fileName); 
                        });
                        for(indexLinkIMG=0;indexLinkIMG<ArrayLinksFileIMG.length;indexLinkIMG++){
                            let pictures = {};
                            pictures["photo"+(indexLinkIMG+1)] = ArrayLinksFileIMG[indexLinkIMG];
                            file = file.toString().supplant(pictures);
                        }
                    }else{}  
                }else{
                    console.log("Erreur");
                }
            });

            response.setHeader("Content-Type", "text/html");
            response.statusCode=200;
            response.end(file);
        }else if (filePath.includes(arrayPages[1][0])){
            // console.log(arrayPages[1][1] + " est demandée")
            buildArrayLinksForAskedPage (resultatTXT, resultatIMG, arrayPages[1][1]);
            // Chargement du contenu dynamique selon la liste de lien contruite précédemment
            
            linkForPage.forEach(link => { 
                // console.log(link);
                if (link.match(regexTXT)){
                    let text = {};
                    text[arrayLink[4][2]] = CNF_FS.readFileSync(`./PUBLIC/${link}`, {encoding: "UTF-8"}).toString();
                    file = file.toString().supplant(text);
                }else if (link.match(regexIMG)){
                    let ListFileInStorageFolder = CNF_FS.readdirSync(`./PUBLIC${link}`, {encoding: "UTF-8"});
                    if (ListFileInStorageFolder != ""){
                        let ArrayLinksFileIMG = [];
                        ListFileInStorageFolder.forEach(fileName => {
                            ListFileInStorageFolder.fileName = `${link}/${fileName}`;
                            ArrayLinksFileIMG.push(ListFileInStorageFolder.fileName); 
                        });
                        for(indexLinkIMG=0;indexLinkIMG<ArrayLinksFileIMG.length;indexLinkIMG++){
                            let pictures = {};
                            pictures["photoH"+(indexLinkIMG+1)] = ArrayLinksFileIMG[indexLinkIMG];
                            pictures["photoB"+(indexLinkIMG+1)] = ArrayLinksFileIMG[indexLinkIMG];
                            file = file.toString().supplant(pictures);
                        }
                    }else{}  
                }else{
                    console.log("Erreur");
                }
            });
            response.setHeader("Content-Type", "text/html");
            response.statusCode=200;
            response.end(file);
        }else if (filePath.includes(arrayPages[2][0])){
            // console.log(arrayPages[2][1] + " est demandée");
            buildArrayLinksForAskedPage (resultatTXT, resultatIMG, arrayPages[2][1]);
            // console.log(linkForPage);
            linkForPage.forEach(link => { 
                // console.log(link);
                if (link.match(regexTXT)) {
                    
                }else if (link.match(regexIMG)){
                    let ListFileInStorageFolder = CNF_FS.readdirSync(`./PUBLIC${link}`, {encoding: "UTF-8"});
                    if (ListFileInStorageFolder != ""){
                        let ArrayLinksFileIMG = [];
                        ListFileInStorageFolder.forEach(fileName => {
                            ListFileInStorageFolder.fileName = `${link}/${fileName}`;
                            ArrayLinksFileIMG.push(ListFileInStorageFolder.fileName); 
                        });
                        for(indexLinkIMG=0;indexLinkIMG<ArrayLinksFileIMG.length;indexLinkIMG++){
                            let pictures = {};
                            pictures["photo"+(indexLinkIMG+1)] = ArrayLinksFileIMG[indexLinkIMG];
                            file = file.toString().supplant(pictures);
                        }
                    }else{}  
                }else{
                    console.log("Erreur");
                }
            });
            response.setHeader("Content-Type", "text/html");
            response.statusCode=200;
            response.end(file);
        }else{
            file = CNF_FS.readFileSync("." + filePath, {encoding: "UTF-8"});
            file = file.toString().supplant(sharedFiles);
            response.setHeader("Content-Type", "text/html");
            response.statusCode=200;
            response.end(file);
        }    
    } catch (error) {
        //console.log(error);
        if (CNF_FS.existsSync('./PUBLIC/ERREUR/404.html', {encoding: "UTF-8"})) {
            file = CNF_FS.readFileSync('./PUBLIC/ERREUR/404.html', {encoding: "UTF-8"});
            file = file.toString().supplant(sharedFiles);
            //console.log(file);
            response.setHeader("Content-Type", "text/html");
            response.statusCode=404;
            return response.end(file);
        } else {
            response.setHeader("Content-Type", "text/html");
            response.statusCode=404;
            return response.end("Erreur 404");
        }
    }
}

/**
 * 
 * // CNF = CoreNodeFoncts
 * No Module
 * 
 * @param {String} Str - Your string where you would like remove space
 * @param {Array} valueToReplace - Your new character or string
 * @param {Array} remplacementValue - Your new character or string
 * @returns 
 */
function removeCharacter(Str,valueToReplace, remplacementValue){

    // Liste caratère encodés avec l'URL Encoding isse de lURL.
    // Attention à entrer tous les caractères spéciaux indésirables du clavier. 
    
    while(Str.includes(valueToReplace)){
        Str = Str.replace(valueToReplace,remplacementValue);
    }
    return Str;
}

/**
 * 
 * // CNF = CoreNodeFoncts
 * No Module
 * 
 * @param {Int} myInt - Your number
 * @param {Int} basenumber - This value could be take 2, 10, 16. The common value is 10.
 * @returns 
 */
 function formatInt(myInt, basenumber){
    myInt = parseInt(myInt, basenumber);
    if (typeof(myInt)=="number"){
        return myInt;
    }else{
        console.log("Erreur");
    }
}

/**
 * 
 * // CNF = CoreNodeFoncts
 * No Module
 * 
 * @param {String} MyStr 
 * @returns - The first letter is written with a UpperCase and last letters are written in lowercases
 */
 function formatAString (myStr){
    let firstLetter;
    let lastLetters;
        
    firstLetter = myStr.substring(0,1);
    lastLetters = myStr.substring(1);
    firstLetter = firstLetter.toUpperCase();
    lastLetters = lastLetters.toLowerCase();
    myStr = firstLetter + lastLetters;
    return myStr;
}

/**
 * 
 * @param {String} myPhone - myPhone could be encoded with special characters.
 */
 function formatAPhoneNumber(myPhone){
    const goodRegex = /^(0[1-9]{1})+([ ]+[0-9]{2}){4}$/; // only XX XX XX XX XX
    const badRegex = [
        /^(0+[1-9]{1})+([0-9]){8}$/,    // 0689835235
        /^\+(33)+([1-9]{1})+([0-9])$/,  //+63XXXXXXXXX ou +33 (X) XX XX XX XX ou +33 X XX XX XX XX
        /^(33)+([1-9]{1})+([0-9])$/     // 33689835235 ou 33 (X) XX XX XX XX ou 33 X XX XX XX XX
    ];
    // Liste caratère encodés avec l'URL Encoding isse de lURL.
    // Attention à entrer tous les caractères spéciaux indésirables du clavier.
    // Affichage Valeur Saisie par l'utilisateur
    // console.log(`Display origin value of "myPhone" : ${myPhone}`);
    let contentTelValueEncoding = [
        "%2B", "%28", "%29",
        
        "a", "b", "c", "d", "e", "f", "g", "h", "i", "j",
        "k", "l", "m", "n", "o", "p", "q", "r", "s", "t",
        "u", "v", "w", "x", "y", "z",
        "A", "B", "C", "D", "E", "F", "G", "H", "I", "J",
        "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T",
        "U", "V", "W", "W", "X", "Y", "Z",
        "%21", "%22", "%23", "%24", "%25", "%26", "%27",
        "%2A", "%2B", "%2C", "%2D", "%2E", "%2F", "%3A",
        "%3B", "%3C", "%3D", "%3E", "%3F", "%40", "%5B",
        "%5C", "%5D", "%5E", "%5F", "%60", "%7B", "%7C", 
        "%7D", "%7E", "%7F", "%E2%82%AC", "%E2%80%9A",
        "%C6%92", "%E2%80%9E", "%E2%80%A6", "%E2%80%A0",
        "%E2%80%A1", "%CB%86", "%E2%80%B0", "%C5%A0", 
        "%E2%80%B9", "%C5%92", "%C5%8D", "%C5%BD", "%8F",
        "%C2%90", "%E2%80%98", "%E2%80%99", "%E2%80%9C",
        "%E2%80%9D", "%E2%80%A2", "%E2%80%93", "%E2%80%94",
        "%CB%9C", "%E2%84", "%C5%A1", "%E2%80", "%C5%93", "%9D",
        "%C5%BE", "%C5%B8", "%C2%A0", "%C2%A1", "%C2%A2", "%C2%A3",
        "%C2%A4", "%C2%A5", "%C2%A6", "%C2%A7", "%C2%A8", "%C2%A9",
        "%C2%AA", "%C2%AB", "%C2%AC", "%C2%AD", "%C2%AE", "%C2%AF",
        "%C2%B0", "%C2%B1", "%C2%B2", "%C2%B3", "%C2%B4", "%C2%B5",
        "%C2%B6", "%C2%B7", "%C2%B8", "%C2%B09", "%C2%BA","%C2%BB",
        "%C2%BC", "%C2%BD", "%C2%BE", "%C2%BF", "%C3%80", "%C3%81",
        "%C3%82", "%C3%83", "%C3%84", "%C3%85", "%C3%86", "%C3%87",
        "%C3%88", "%C3%89", "%C3%8A", "%C3%8B", "%C3%8C", "%C3%8D",
        "%C3%8E", "%C3%8F", "%C3%90", "%C3%91", "%C3%92", "%C3%93",
        "%C3%94", "%C3%95", "%C3%96", "%C3%97", "%C3%98", "%C3%99",
        "%C3%9A", "%C3%9B", "%C3%9C", "%C3%9D", "%C3%9E", "%C3%9F",
        "%C3%A0", "%C3%A1", "%C3%A2", "%C3%A4", "%C3%A5", "%C3%A6",
        "%C3%A7", "%C3%A8", "%C3%A9", "%C3%AA", "%C3%AB", "%C3%AC",
        "%C3%AD", "%C3%AE", "%C3%AF", "%C3%B0", "%C3%B1", "%C3%B2",
        "%C3%B3", "%C3%B4", "%C3%B5", "%C3%B6", "%C3%B7", "%C3%B8",
        "%C3%B9", "%C3%BA", "%C3%BB", "%C3%BC", "%C3%BD", "%C3%BE",
        "%C3%BF"
    ];
    // Pour chaque caractère spécial présent dans la valeur de "myPhone" appeler la fonction supppression d'espace.

    contentTelValueEncoding.forEach(element => {
        myPhone = removeCharacter(myPhone,element,"");
    });
    // console.log("Before Regex : "+myPhone);
    while (!myPhone.match(goodRegex)){
        if (myPhone.match(badRegex[0])){
            //console.log(`\"${myPhone} : 1"`);
            myPhone = `${myPhone.substring(0,2)} ${myPhone.substring(2,4)} ${myPhone.substring(4,6)} ${myPhone.substring(6,8)} ${myPhone.substring(8,10)}`
            return myPhone;
        }else if (myPhone.match(badRegex[1])){
            myPhone = `\"0${myPhone.substring(3)}"`;
            //console.log(myPhone + " : 2");
            return myPhone;
        }else if (myPhone.match(badRegex[2])){
            myPhone = `\"0${myPhone.substring(2,3)} ${myPhone.substring(3,5)} ${myPhone.substring(5,7)} ${myPhone.substring(7,9)} ${myPhone.substring(9,11)}"`;
            //console.log(myPhone + " : 3");
            return myPhone;
        }else{
            return '';
        }
    }
    if (myPhone.match(goodRegex)){
        return myPhone;
    }
}

module.exports = {
    fonctSendFile:sendFile,
    fonctSendHTMLFile:sendHTMLFile,
    fonctSupprimerCharactère:removeCharacter,
    fonctFormatString:formatAString,
    fonctFormatInt:formatInt,
    fonctFormatPhoneNumber:formatAPhoneNumber
}